/***********************************************************************
/ Movable measurement cursors for OpenSCAD
/ Done in customizer as there is no viewport object drag support
/ Usage:
/ In your script, paste in the customizables section from cursors.scad,
/ then uncomment the lines
/ // include <cursors.scad>
/ // cursors();
/ and move them by dragging the Cursor sliders in the customizer.
/ For 2D cursors, use cursors2d() instead of cursors().
/ In Preview mode, measurements are reported in the console if curs0, 
/ curs1 and cursbetween are enabled. 
/ All cursors are disabled in Render.
***********************************************************************/


// Customizables - copy to your script and uncomment
/*[Cursors]*/
/* //uncomment here
curs0=true;
xcur0=-5;//[-50:0.05:50]
ycur0=-5;//[-50:0.05:50]
zcur0=-5;//[-50:0.05:50]
curs1=true;
xcur1=5;//[-50:0.05:50]
ycur1=5;//[-50:0.05:50]
zcur1=5;//[-50:0.05:50]
cursbetween=true;

cursize=3;
curthick=0.05;

curcolors=[["red","green","blue"],["orange","lime","aqua"],"black"];
curopacity=0.5;
*/ //uncomment here

// End customizables
// use <cursors.scad>
// cursors();


module _cursor(pozn,colours){
    translate(pozn) {
        color(colours.x,curopacity) cube([cursize,curthick,curthick],center=true);
        color(colours.y,curopacity) cube([curthick,cursize,curthick],center=true);
        color(colours.z,curopacity) cube([curthick,curthick,cursize],center=true);
    }
}

module cursors(){
    if($preview&&(curs0==true)) _cursor([xcur0,ycur0,zcur0],curcolors[0]);
    if($preview&&(curs1==true)) _cursor([xcur1,ycur1,zcur1],curcolors[1]);
    if($preview&&(curs0==true)&&(curs1==true)&&(cursbetween==true)){
        color(curcolors[2],curopacity) 
            hull(){
                translate([xcur0,ycur0,zcur0]) cube(curthick,center=true);
                translate([xcur1,ycur1,zcur1]) cube(curthick,center=true);
            }
        echo(str("\n\n", //Formatted console output
            
            "*** Cursors ***\n",
            "Xdiff: ",(xcur1-xcur0),"\n",
            "Ydiff: ",(ycur1-ycur0),"\n",
            "Zdiff: ",(zcur1-zcur0),"\n",
            "Distance: ",sqrt(((xcur1-xcur0)^2)+((ycur1-ycur0)^2)+((zcur1-zcur0)^2)),"\n",
            "Azimuth: ",atan2((ycur1-ycur0),(xcur1-xcur0)),"°\n",
            "Elevation: ",atan2((zcur1-zcur0),sqrt(((xcur1-xcur0)^2)+((ycur1-ycur0)^2))),"°\n",
            
            "\n\n"));
    }
}


module _cursor2d(pozn,colours){
    translate(pozn) {
        color(colours.x,curopacity) square([cursize,curthick],center=true);
        color(colours.y,curopacity) square([curthick,cursize],center=true);
    }
}
module cursors2d(){
    if($preview&&(curs0==true)) _cursor2d([xcur0,ycur0],curcolors[0]);
    if($preview&&(curs1==true)) _cursor2d([xcur1,ycur1],curcolors[1]);
    if($preview&&(curs0==true)&&(curs1==true)&&(cursbetween==true)){
        color(curcolors[2],curopacity) 
            hull(){
                translate([xcur0,ycur0]) square(curthick,center=true);
                translate([xcur1,ycur1,]) square(curthick,center=true);
            }
        echo(str("\n\n", //Formatted console output
            
            "*** Cursors ***\n",
            "Xdiff: ",(xcur1-xcur0),"\n",
            "Ydiff: ",(ycur1-ycur0),"\n",
            "Distance: ",sqrt(((xcur1-xcur0)^2)+((ycur1-ycur0)^2)),"\n",
            "Azimuth: ",atan2((ycur1-ycur0),(xcur1-xcur0)),"°\n",           
            "\n\n"));
    }
}

// cursors(); //Demo only